@@ -61,11 +61,16 @@ fm_bbox.numeric <- function(x, ...) {
 fm_bbox.matrix <- function(x, ...) {
   do.call(
     c,
-    c(lapply(seq_len(ncol(x)),
-             function(k) {
-               fm_bbox(x[, k, drop = TRUE])
-             }),
-      list(.join = TRUE)))
+    c(
+      lapply(
+        seq_len(ncol(x)),
+        function(k) {
+          fm_bbox(x[, k, drop = TRUE])
+        }
+      ),
+      list(.join = TRUE)
+    )
+  )
 }
 
 #' @rdname fm_bbox
@@ -73,11 +78,16 @@ fm_bbox.matrix <- function(x, ...) {
 fm_bbox.Matrix <- function(x, ...) {
   do.call(
     c,
-    c(lapply(seq_len(ncol(x)),
-             function(k) {
-               fm_bbox(x[, k, drop = TRUE])
-             }),
-      list(.join = TRUE)))
+    c(
+      lapply(
+        seq_len(ncol(x)),
+        function(k) {
+          fm_bbox(x[, k, drop = TRUE])
+        }
+      ),
+      list(.join = TRUE)
+    )
+  )
 }
 
 #' @rdname fm_bbox
