@@ -1,7 +1,7 @@
 #' @include deprecated.R
 #' @include print.R
 
-#' @importFrom sp coordinates proj4string `proj4string<-`
+#' @importFrom sp coordinates proj4string proj4string<-
 
 
 #' @title Handling CRS/WKT
@@ -1178,6 +1178,18 @@ fm_CRS.fm_CRS <- function(x, oblique = NULL, ...) {
   x
 }
 
+#' @rdname fm_CRS_sp
+#' @export
+fm_CRS.SpatVector <- function(x, oblique = NULL, ...) {
+  fm_CRS(fm_crs(x, ...), oblique = oblique)
+}
+
+#' @rdname fm_CRS_sp
+#' @export
+fm_CRS.SpatRaster <- function(x, oblique = NULL, ...) {
+  fm_CRS(fm_crs(x, ...), oblique = oblique)
+}
+
 #' @rdname fm_CRS_sp
 #' @export
 fm_CRS.sf <- function(x, oblique = NULL, ...) {
