@@ -115,13 +115,12 @@ std::istream &operator>>(std::istream &input, IOHeader &h) {
 }
 
 IOHelperC &IOHelperC::OL(std::ostream &output) {
-  const IOHeader &h(IOHelper<int>::h_);
-  const bool &bin_(IOHelper<int>::binary_);
+  const IOHeader &h(IOHelper<int>::header());
   if ((h.elems == 0) || (!cM_)) {
     return *this;
   }
   for (const auto& outi : cM_->output_) {
-    if (bin_) {
+    if (is_binary()) {
       int string_size = outi.length() + 1;
       output.write((char *)&string_size, sizeof(string_size));
       output.write((char *)outi.c_str(), sizeof(char) * string_size);
@@ -133,8 +132,8 @@ IOHelperC &IOHelperC::OL(std::ostream &output) {
 }
 
 IOHelperC &IOHelperC::IL(std::istream &input) {
-  const IOHeader &h(IOHelper<int>::h_);
-  const bool &bin_(IOHelper<int>::binary_);
+  const IOHeader &h(IOHelper<int>::header());
+  const bool bin_(IOHelper<int>::is_binary());
   if ((h.elems == 0) || (!cM_)) {
     return *this;
   }
@@ -156,8 +155,8 @@ IOHelperC &IOHelperC::IL(std::istream &input) {
 }
 
 IOHelperC &IOHelperC::OD(std::ostream &output) {
-  const IOHeader &h(IOHelper<int>::h_);
-  const bool &bin_(IOHelper<int>::binary_);
+  const IOHeader &h(IOHelper<int>::header());
+  const bool bin_(IOHelper<int>::is_binary());
   if ((h.elems == 0) || (!cM_)) {
     return *this;
   }
@@ -166,20 +165,20 @@ IOHelperC &IOHelperC::OD(std::ostream &output) {
     if (mcc.info.datatype == IODatatype::Dense)
       if (mcc.info.valuetype == IOValuetype::Int) {
         IOHelperM<int> ioh;
-        ioh.cD(&(mcc.DI())).matrixtype(mcc.info.matrixtype);
+        ioh.cD(mcc.cDI()).matrixtype(mcc.info.matrixtype);
         ioh.binary(bin_).OH(output).OD(output);
       } else {
         IOHelperM<double> ioh;
-        ioh.cD(&mcc.DD()).matrixtype(mcc.info.matrixtype);
+        ioh.cD(mcc.cDD()).matrixtype(mcc.info.matrixtype);
         ioh.binary(bin_).OH(output).OD(output);
       }
     else if (mcc.info.valuetype == IOValuetype::Int) {
       IOHelperSM<int> ioh;
-      ioh.cD(&mcc.SI()).matrixtype(mcc.info.matrixtype);
+      ioh.cD(mcc.cSI()).matrixtype(mcc.info.matrixtype);
       ioh.binary(bin_).OH(output).OD(output);
     } else {
       IOHelperSM<double> ioh;
-      ioh.cD(&mcc.SD()).matrixtype(mcc.info.matrixtype);
+      ioh.cD(mcc.cSD()).matrixtype(mcc.info.matrixtype);
       ioh.binary(bin_).OH(output).OD(output);
     }
   }
@@ -187,8 +186,8 @@ IOHelperC &IOHelperC::OD(std::ostream &output) {
 }
 
 IOHelperC &IOHelperC::ID(std::istream &input) {
-  const IOHeader &h(IOHelper<int>::h_);
-  const bool &bin_(IOHelper<int>::binary_);
+  const IOHeader &h(IOHelper<int>::header());
+  const bool bin_(IOHelper<int>::is_binary());
   if ((!M_) || (h.elems == 0)) {
     return *this;
   }
@@ -197,24 +196,24 @@ IOHelperC &IOHelperC::ID(std::istream &input) {
     IOHelper<int> ioh_;
     ioh_.binary(bin_).IH(input);
 
-    if (ioh_.h_.datatype == IODatatype::Dense)
-      if (ioh_.h_.valuetype == IOValuetype::Int) {
+    if (ioh_.header().datatype == IODatatype::Dense)
+      if (ioh_.header().valuetype == IOValuetype::Int) {
         IOHelperM<int> ioh;
         ioh.D(&(M_->DI(listi)));
-        ioh.binary(bin_).IH(ioh_.h_).ID(input);
+        ioh.binary(bin_).IH(ioh_.header()).ID(input);
       } else {
         IOHelperM<double> ioh;
         ioh.D(&(M_->DD(listi)));
-        ioh.binary(bin_).IH(ioh_.h_).ID(input);
+        ioh.binary(bin_).IH(ioh_.header()).ID(input);
       }
-    else if (ioh_.h_.valuetype == IOValuetype::Int) {
+    else if (ioh_.header().valuetype == IOValuetype::Int) {
       IOHelperSM<int> ioh;
       ioh.D(&(M_->SI(listi)));
-      ioh.binary(bin_).IH(ioh_.h_).ID(input);
+      ioh.binary(bin_).IH(ioh_.header()).ID(input);
     } else {
       IOHelperSM<double> ioh;
       ioh.D(&(M_->SD(listi)));
-      ioh.binary(bin_).IH(ioh_.h_).ID(input);
+      ioh.binary(bin_).IH(ioh_.header()).ID(input);
     }
   }
   return *this;
@@ -225,43 +224,83 @@ IOHelperC &IOHelperC::ID(std::istream &input) {
 
 
 template <>
-Matrix<int> &MatrixC::attach(std::string name, Matrix<int> *M,
-                             bool transfer_ownership, IOMatrixtype matrixt) {
+Matrix<int> &MatrixC::attach(std::string name,
+                             Matrix<int> *M,
+                             IOMatrixtype matrixt) {
   free(name);
-  coll_.insert(collPairT(name, new MCC(IODatatype::Dense, IOValuetype::Int,
-                                       matrixt, M, transfer_ownership)));
+  coll_.insert(collPairT(name, std::make_unique<MCC>(M, matrixt, false)));
   activate(name);
   return coll_[name]->DI();
 }
 
+
 template <>
-Matrix<double> &MatrixC::attach(std::string name, Matrix<double> *M,
-                                bool transfer_ownership, IOMatrixtype matrixt) {
+Matrix<double> &MatrixC::attach(std::string name,
+                                Matrix<double> *M,
+                                IOMatrixtype matrixt) {
   free(name);
-  coll_.insert(collPairT(name, new MCC(IODatatype::Dense, IOValuetype::Double,
-                                       matrixt, M, transfer_ownership)));
+  coll_.insert(collPairT(name, std::make_unique<MCC>(M, matrixt, false)));
   activate(name);
   return coll_[name]->DD();
 }
 
+
 template <>
-SparseMatrix<int> &MatrixC::attach(std::string name, SparseMatrix<int> *M,
-                                   bool transfer_ownership,
+SparseMatrix<int> &MatrixC::attach(std::string name,
+                                   SparseMatrix<int> *M,
                                    IOMatrixtype matrixt) {
   free(name);
-  coll_.insert(collPairT(name, new MCC(IODatatype::Sparse, IOValuetype::Int,
-                                       matrixt, M, transfer_ownership)));
+  coll_.insert(collPairT(name, std::make_unique<MCC>(M, matrixt, false)));
   activate(name);
   return coll_[name]->SI();
 }
 
 template <>
-SparseMatrix<double> &MatrixC::attach(std::string name, SparseMatrix<double> *M,
-                                      bool transfer_ownership,
+SparseMatrix<double> &MatrixC::attach(std::string name,
+                                      SparseMatrix<double> *M,
                                       IOMatrixtype matrixt) {
   free(name);
-  coll_.insert(collPairT(name, new MCC(IODatatype::Sparse, IOValuetype::Double,
-                                       matrixt, M, transfer_ownership)));
+  coll_.insert(collPairT(name, std::make_unique<MCC>(M, matrixt, false)));
+  activate(name);
+  return coll_[name]->SD();
+}
+
+template <>
+Matrix<int> &MatrixC::attach(std::string name,
+                             std::unique_ptr<Matrix<int>>&& M,
+                             IOMatrixtype matrixt) {
+  free(name);
+  coll_.insert(collPairT(name, std::make_unique<MCC>(std::move(M), matrixt)));
+  activate(name);
+  return coll_[name]->DI();
+}
+
+template <>
+Matrix<double> &MatrixC::attach(std::string name,
+                                std::unique_ptr<Matrix<double>>&& M,
+                                IOMatrixtype matrixt) {
+  free(name);
+  coll_.insert(collPairT(name,std::make_unique<MCC>(std::move(M), matrixt)));
+  activate(name);
+  return coll_[name]->DD();
+}
+
+template <>
+SparseMatrix<int> &MatrixC::attach(std::string name,
+                                   std::unique_ptr<SparseMatrix<int>>&& M,
+                                   IOMatrixtype matrixt) {
+  free(name);
+  coll_.insert(collPairT(name, std::make_unique<MCC>(std::move(M), matrixt)));
+  activate(name);
+  return coll_[name]->SI();
+}
+
+template <>
+SparseMatrix<double> &MatrixC::attach(std::string name,
+                                      std::unique_ptr<SparseMatrix<double>>&& M,
+                                      IOMatrixtype matrixt) {
+  free(name);
+  coll_.insert(collPairT(name, std::make_unique<MCC>(std::move(M), matrixt)));
   activate(name);
   return coll_[name]->SD();
 }
@@ -338,9 +377,11 @@ MCCInfo MatrixC::load(std::string name) {
   ioh_.binary(bin_in_).IH(I);
   I.close();
 
-  coll_.insert(collPairT(name, new MCC(IODatatype(ioh_.h_.datatype),
-                                       IOValuetype(ioh_.h_.valuetype),
-                                       IOMatrixtype(ioh_.h_.matrixtype))));
+  coll_.insert(collPairT(
+      name,
+      std::make_unique<MCC>(IODatatype(ioh_.header().datatype),
+                            IOValuetype(ioh_.header().valuetype),
+                            IOMatrixtype(ioh_.header().matrixtype))));
   activate(name);
 
   I.open((input_prefix_ + name).c_str(),
@@ -348,8 +389,8 @@ MCCInfo MatrixC::load(std::string name) {
   if (!I.is_open()) {
     return info(name);
   }
-  if (ioh_.h_.datatype == IODatatype::Dense)
-    if (ioh_.h_.valuetype == IOValuetype::Int) {
+  if (ioh_.header().datatype == IODatatype::Dense)
+    if (ioh_.header().valuetype == IOValuetype::Int) {
       IOHelperM<int> ioh;
       ioh.D(&DI(name));
       ioh.binary(bin_in_).IH(I);
@@ -359,7 +400,7 @@ MCCInfo MatrixC::load(std::string name) {
       ioh.D(&DD(name));
       ioh.binary(bin_in_).IH(I).ID(I);
     }
-  else if (ioh_.h_.valuetype == IOValuetype::Int) {
+  else if (ioh_.header().valuetype == IOValuetype::Int) {
     IOHelperSM<int> ioh;
     ioh.D(&SI(name));
     ioh.binary(bin_in_).IH(I).ID(I);
@@ -380,7 +421,6 @@ MatrixC &MatrixC::free(std::string name) {
 
   collT::iterator colli;
   if ((colli = coll_.find(name)) != coll_.end()) {
-    delete colli->second;
     coll_.erase(colli);
   }
   return *this;
@@ -516,7 +556,7 @@ Matrix<int> &MatrixC::DI(std::string name) {
       (colli->second->info.active)) {
     return colli->second->DI();
   }
-  return attach(name, new Matrix<int>());
+  return attach(name, std::make_unique<Matrix<int>>());
 }
 
 Matrix<double> &MatrixC::DD(std::string name) {
@@ -527,7 +567,7 @@ Matrix<double> &MatrixC::DD(std::string name) {
       (colli->second->info.active)) {
     return colli->second->DD();
   }
-  return attach(name, new Matrix<double>());
+  return attach(name, std::make_unique<Matrix<double>>());
 }
 
 SparseMatrix<int> &MatrixC::SI(std::string name) {
@@ -538,7 +578,7 @@ SparseMatrix<int> &MatrixC::SI(std::string name) {
       (colli->second->info.active)) {
     return colli->second->SI();
   }
-  return attach(name, new SparseMatrix<int>());
+  return attach(name, std::make_unique<SparseMatrix<int>>());
 }
 
 SparseMatrix<double> &MatrixC::SD(std::string name) {
@@ -549,7 +589,7 @@ SparseMatrix<double> &MatrixC::SD(std::string name) {
       (colli->second->info.active)) {
     return colli->second->SD();
   }
-  return attach(name, new SparseMatrix<double>());
+  return attach(name, std::make_unique<SparseMatrix<double>>());
 }
 
 void MatrixC::matrixtype(std::string name, IOMatrixtype matrixt) {
@@ -574,14 +614,24 @@ SEXP MatrixC::Rcpp_wrap() const {
   for (auto const & outi : output_) {
     const MCC &mcc = *(coll_.find(outi)->second);
     if (mcc.info.datatype == IODatatype::Dense) {
-      if (mcc.info.valuetype == IOValuetype::Int)
-        res[outi] = Rcpp::wrap(mcc.DI());
-      else
-        res[outi] = Rcpp::wrap(mcc.DD());
+      if (mcc.info.valuetype == IOValuetype::Int) {
+        if (auto ptr = mcc.cDI()) {
+          res[outi] = Rcpp::wrap(*ptr);
+        }
+      } else {
+        if (auto ptr = mcc.cDD()) {
+          res[outi] = Rcpp::wrap(*ptr);
+        }
+      }
     } else if (mcc.info.valuetype == IOValuetype::Int) {
-      res[outi] = Rcpp::wrap(mcc.SI());
-    } else
-      res[outi] = Rcpp::wrap(mcc.SD());
+      if (auto ptr = mcc.cSI()) {
+        res[outi] = Rcpp::wrap(*ptr);
+      }
+    } else {
+      if (auto ptr = mcc.cSD()) {
+        res[outi] = Rcpp::wrap(*ptr);
+      }
+    }
   }
   return res;
 }
@@ -589,31 +639,30 @@ SEXP MatrixC::Rcpp_wrap() const {
 
 void MatrixC::attach(std::string name, SEXP from) {
   if (Rcpp::is<Rcpp::NumericMatrix>(from)) {
-    (*this).attach(name, new Matrix<double>(
+    (*this).attach(name, std::make_unique<Matrix<double>>(
         Rcpp::as<Rcpp::NumericMatrix>(
-          from)), true,
+          from)),
           IOMatrixtype::General);
   } else if (Rcpp::is<Rcpp::IntegerMatrix>(from)) {
-    (*this).attach(name, new Matrix<int>(
+    (*this).attach(name, std::make_unique<Matrix<int>>(
         Rcpp::as<Rcpp::IntegerMatrix>(
-          from)), true,
+          from)),
           IOMatrixtype::General);
   } else if (Rcpp::is<Rcpp::CharacterMatrix>(from)) {
   } else if (Rcpp::is<Rcpp::NumericVector>(from)) {
-    (*this).attach(name, new Matrix1<double>(
+    (*this).attach(name, std::make_unique<Matrix<double>>(
         Rcpp::as<Rcpp::NumericVector>(
-          from)), true,
+          from)),
           IOMatrixtype::General);
   } else if (Rcpp::is<Rcpp::IntegerVector>(from)) {
-    (*this).attach(name, new Matrix1<int>(
+    (*this).attach(name, std::make_unique<Matrix<int>>(
         Rcpp::as<Rcpp::IntegerVector>(
-          from)), true,
+          from)),
           IOMatrixtype::General);
   } else if (Rcpp::is<Rcpp::CharacterVector>(from)) {
   } else {
     (*this).attach(name,
-     new SparseMatrix<double>(from),
-     true,
+     std::make_unique<SparseMatrix<double>>(from),
      IOMatrixtype::General);
   }
 }
